package tantra;

/**
 * チャットログのファイル名に付いている日付データを取り扱うためのクラス。
 * 
 * @author Luft
 * 
 */
public class FileSpecifier {

	/** ChatLogが記録された年 */
	private int year = 0; 
	/** ChatLogが記録された月 */
	private int month = 0;
	/** ChatLogが記録された日 */
	private int day = 0;
	/** ChatLogが記録された番号 */
	private int index = 0;


	/**
	 * 引数なしコンストラクタ。
	 */
	public FileSpecifier() {
		year = 0;
		month = 0;
		day = 0;
		index = 0;
	}


	/**
	 * 引数ありコンストラクタ。
	 * 
	 * @param ayear
	 *        年
	 * @param amonth
	 *        月
	 * @param aday
	 *        日
	 * @param aindex
	 *        インデックス
	 */
	public FileSpecifier( int ayear, int amonth, int aday, int aindex ) {
		year = ayear;
		month = amonth;
		day = aday;
		index = aindex;
	}


	/**
	 * ファイル名からのコンストラクタ。年月日とその日のログファイルのインデックスを取り出します
	 * 
	 * @param fname
	 *        対象となるファイルの名前
	 */
	public FileSpecifier( String fname ) {
		if( fname.equals( "" ) ){
			year = 0;
			month = 0;
			day = 0;
			index = 0;
		}

		fname = fname.replace( ".txt", "" );
		String token[];
		token = fname.split( "_" );

		if( token[0].equals( "ChatLog" ) ){
			year = Integer.valueOf( token[1] ).intValue();
			month = Integer.valueOf( token[2] ).intValue();
			day = Integer.valueOf( token[3] ).intValue();
			index = Integer.valueOf( token[4] ).intValue();
		}else{
			year = 0;
			month = 0;
			day = 0;
			index = 0;
		}
	}


	/**
	 * このFileSpecifierがfsより大きいかどうかを判定します。
	 * 
	 * @param fs
	 *        比較対象のFileSpecifier
	 * @return このFileSpecifierが<code>fs</code>よりも新しいものを示していればtrueを、
	 *         そうでなければfalseを返します。
	 */
	public boolean largerThan( FileSpecifier fs ){
		int ymd1, ymd2;
		ymd1 = ymd();
		ymd2 = fs.ymd();
		if( ymd1 == ymd2 ){
			if( index > fs.index ){
				return true;
			}else{
				return false;
			}
		}else{
			if( ymd1 > ymd2 ){
				return true;
			}else{
				return false;
			}
		}
	}


	/**
	 * このFileSpecifierが、<code>fs</code>と等しいかどうかを判定します。
	 * 
	 * @param fs
	 *        比較対象のFileSpecifier
	 * @return 等しければtrueを、そうでなければfalseを返します。
	 */
	public boolean equals( FileSpecifier fs ){
		if( year == fs.year && month == fs.month && day == fs.day
				&& index == fs.index ){
			return true;
		}else{
			return false;
		}
	}


	/**
	 * オブジェクトが空かどうか判定します
	 * 
	 * @return 空なら<code>true</code>を、そうでなければ<code>false</code>を返します
	 */
	public boolean isNull(){
		FileSpecifier nul = new FileSpecifier();
		if( this.equals( nul ) ){
			return true;
		}else{
			return false;
		}
	}


	/**
	 * この<code>FileSpecifier</code>の
	 * <code>year * 10000 + month * 100 + day</code>の値を求めます。
	 * 
	 * @return <code>year * 10000 + month * 100 + day</code>の値
	 */
	public int ymd(){
		return year * 10000 + month * 100 + day;
	}


	/**
	 * この<code>FileSpecifier</code>が表すファイルの名前を返します。
	 * 
	 * @return ファイル名。ex."ChatLog_2007_11_7_0.txt"
	 */
	public String name(){
		return "ChatLog_" + year + "_" + month + "_" + day + "_" + index
				+ ".txt";
	}

}
