package tantra;

/**
 * Tantraの種族名などの定数を提供。種族を表す整数値から、種族名の文字列を取得するなど。 
 * 
 * @author Luft
 * 
 */
public enum Job {
	/** アス・ラク */
	Asura_Rakshasa(0),
	/** ディバ・ガル */
	Deva_Garuda(1),
	/** ナガ・キン */
	Naga_Kimnara(2),
	/** ヤク・ガン */
	Yaksa_Gandharva(3),
	/** 不明 */
	Unknown(4);

	/** Asura_Rakshasaのvalue値 */
	private final static int ar = 0;
	/** Deva_Garudaのvalue値 */
	private final static int dg = 1;
	/** Naga_Kimnaraのvalue値 */
	private final static int nk = 2;
	/** Yaksa_Gandharvaのvalue値 */
	private final static int yg = 3;

	/** value値。ar, dg, nk, ygのいずれかが代入される */
	private final int value;

	/** "Asura_Rakshasa" */
	public final static String ASURA_RAKSHASA = "Asura_Rakshasa";
	/** "Deva_Garuda" */
	public final static String DEVA_GARUDA = "Deva_Garuda";
	/** "Naga_Kimnara" */
	public final static String NAGA_KIMNARA = "Naga_Kimnara";
	/** "Yaksa_Gandharava" */
	public final static String YAKSA_GANDHARVA = "Yaksa_Gandharva";
	/** "Unknown" */
	public final static String UNKNOWN = "Unknown";


	/**
	 * プライベートなコンストラクタ
	 * 
	 * @param value
	 */
	private Job( int value ) {
		this.value = value;
	}


	public static Job fromInt( int value ){
		switch( value ){
			case ar:
				return Job.Asura_Rakshasa;
			case dg:
				return Job.Deva_Garuda;
			case nk:
				return Job.Naga_Kimnara;
			case yg:
				return Job.Yaksa_Gandharva;
			default:
				return Job.Unknown;
		}
	}

	
	/**
	 * 種族判定値に対応する文字列を求めます。
	 * 
	 * @return jobに対応する文字列。"Asura_Rakshasa"など。
	 */
	public String toString(){
		switch( value ){
			case ar:
				return ASURA_RAKSHASA;
			case dg:
				return DEVA_GARUDA;
			case nk:
				return NAGA_KIMNARA;
			case yg:
				return YAKSA_GANDHARVA;
			default:
				return UNKNOWN;
		}
	}


	/**
	 * このインスタンスのvalue値を求めます
	 * @return
	 * 		value値
	 */
	public int getValue(){
		return this.value;
	}

}
