package tantra;

/**
 * Tantraの主神名などの定数などを提供。
 * 
 * @author Luft
 * 
 */
public enum Lord {
	/** ブラフマ */
	Brahma(0),
	/** ヴィシュヌ */
	Vishnu(1),
	/** シヴァ */
	Shiva(2),
	/** 不明 */
	Unknown(3);

	/** value値 */
	private final int value;

	/** Brahmaのvalue値 */
	private final static int b = 0;
	/** Vishnuのvalue値 */
	private final static int v = 1;
	/** Shivaのvalue値 */
	private final static int s = 2;

	/** "ブラフマー" */
	public final static String BRAHMA = "ブラフマー";
	/** "ヴィシュヌ" */
	public final static String VISHNU = "ヴィシュヌ";
	/** "シヴァ" */
	public final static String SHIVA = "シヴァ";


	/**
	 * プライベートなコンストラクタ
	 * @param value
	 */
	private Lord( int value ) {
		this.value = value;
	}

	
	/**
	 * 主神判定値に対応する文字列を求めます。英字版。
	 * 
	 * @param vLord
	 *        判定する種族判定値
	 * @return vLordに対応する文字列。"Brahma"など。
	 */
	public String toStringEn(){
		switch( value ){
			case b:
				return "Brahma";
			case v:
				return "Vishnu";
			case s:
				return "Shiva";
			default:
				return "All";
		}
	}


	/**
	 * 主神判定値に対応する文字列を求めます。邦字版。
	 * 
	 * @param vLord
	 *        判定する種族判定値
	 * @return vLordに対応する文字列。"黄"など。
	 */
	public String toString(){
		switch( value ){
			case b:
				return "黄";
			case v:
				return "赤";
			case s:
				return "青";
			default:
				return "？";
		}
	}

	
	/**
	 * intからの偽コンストラクタ
	 * @param value
	 * @return
	 */
	public static Lord fromInt( int value ){
		switch( value ){
			case b:
				return Brahma;
			case v:
				return Vishnu;
			case s:
				return Shiva;
			default:
				return Unknown;
		}
	}
}
