package tantra;

import java.util.*;

/**
 * FTPサーバーにファイルを送信するためのクラス。短時間の散発的なアップロードを避け、 一定時間ごとにまとめてアップロードさせたりする。
 * 
 * @author Luft
 * 
 */
public class Uploader {

	private String m_sHost;
	private String m_sUser;
	private String m_sRemotePath;
	private String m_sPassWord;
	private Vector<FileEntry> m_sFileList = new Vector<FileEntry>();
	private DateEx m_dLastUploadedDate = new DateEx();
	private int m_iUploadInterval = 300;


	/**
	 * コンストラクタ
	 * 
	 * @param host
	 *        ホストの名前、もしくはIPアドレス。
	 * @param user
	 *        ユーザー名。
	 * @param pass
	 *        パスワード。
	 * @param path
	 *        サーバー上のパス。
	 */
	public Uploader( String host, String user, String pass, String path ) {
		m_sHost = host;
		m_sUser = user;
		m_sPassWord = pass;
		m_sRemotePath = path;
	}


	/**
	 * FTPSettingクラスを使ったコンストラクタ。
	 * 
	 * @param set
	 */
	public Uploader( FTPSetting set ) {
		m_sHost = set.host;
		m_sUser = set.user_id;
		m_sPassWord = set.user_pw;
		m_sRemotePath = set.path_on_server;
	}


	/**
	 * アップロードするファイルを予約します
	 * 
	 * @param fname
	 *        アップロードを予約するファイルの名前。フルパスで指定すること。
	 */
	public void registerUpload( String fname ){
		if( tantra.settings.target_kruma.equals( "debug" ) ) return;
		//System.out.println( "Uploader::registerUpload;fname=" + fname );
		m_sFileList.add( new FileEntry( fname, false ) );
		tryUpload();
	}


	/**
	 * ファイルのアップロードを試行します 前回アップロードを行った時刻からm_iUploadInterval秒以上経過していた
	 * 場合にのみ、アップロード操作を実行します。
	 */
	public void tryUpload(){
		if( m_sFileList.size() <= 0 ) return;
		DateEx d = new DateEx();
		d.now();
		long tspan = d.subtract( m_dLastUploadedDate );
		//System.out.println( "tspan=" + tspan );
		if( tspan >= m_iUploadInterval ){

			// ファイル名の重複を除去する
			int j, k;
			String sk, sj;
			Vector<FileEntry> buffer = new Vector<FileEntry>();
			for( j = 0; j <= m_sFileList.size() - 2; j++ ){
				sj = m_sFileList.elementAt( j ).name;
				for( k = j + 1; k <= m_sFileList.size() - 2; k++ ){
					sk = m_sFileList.elementAt( k ).name;
					if( !sj.equals( sk ) ){
						buffer.add( new FileEntry( sj, false ) );
						break;
					}
				}
			}
			m_sFileList.removeAllElements();

			// m_sFileListに戻す
			for( j = 0; j <= buffer.size() - 1; j++ ){
				m_sFileList.add( buffer.elementAt( j ) );
			}
			buffer.removeAllElements();

			// upload
			try{
				ftp client = new ftp( m_sHost, m_sUser, m_sPassWord, m_sRemotePath );
				if( client.upload( m_sFileList ) ){
					int change = 0;
					change = 1;
					while( change > 0 ){
						change = 0;
						int i;
						for( i = 0; i < m_sFileList.size(); i++ ){
							if( m_sFileList.elementAt( i ).movable ){
								m_sFileList.removeElementAt( i );
								change = 1;
								break;
							}
						}
					}
					m_sFileList.removeAllElements();
					m_dLastUploadedDate.now();
				}
			}catch( Exception e ){
				tantra.printError( e );
			}
		}
	}


	public int getUploadInterval(){
		return m_iUploadInterval;
	}


	public void setUploadInterval( int uploadInterval ){
		m_iUploadInterval = uploadInterval;
	}

}
