package tantra;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.image.*;
import javax.imageio.*;
import tantra.DateEx.*;

/**
 * 一週間分の聖地の人口データを処理するためのクラス。
 * 
 * @author Luft
 * 
 */
public class WeeklyPopulation {

	private final static String DAILY_POPULATION = "daily_population";
	private final static String DAY_OF_WEEK[] = { "Sun", "Mon", "Tue", "Wed",
			"Thu", "Fri", "Sat" };

	private final static int SPACE_DOWN = 30;
	private final static int SPACE_UP = 40;
	private final static int SPACE_LEFT = 40;
	private final static int SPACE_RIGHT = 25;
	private final static int LEN_MEMORI = 3; // 目盛り線の長さピクセル
	private final static float PIXCELS_PER_X = 24.0F; // x軸1単位あたりのピクセル数
	private final static float PIXCELS_PER_Y = 5.0F; // y軸1単位あたりのピクセル数
	private final static Color BACKGROUND_COLOR = new Color( 245, 245, 245 );
	private final static Color BACKGROUND_UNKNOWN = new Color( 180, 180, 180 );

	public Vector<DailyPopulation> weekly_population = new Vector<DailyPopulation>();


	/**
	 * 聖地人口の時間推移を保管してあるweekly_population変数を初期化します
	 * 
	 * @param d
	 *        初期化に使う日付。<code>d</code>が含まれる週の日曜日の日付を使って、初期化されます
	 */
	public void initialize( DateEx d ){
		weekly_population.removeAllElements();
		weekly_population.setSize( 7 );
		DateEx d1 = d.weekStart();
		int i;
		for( i = 0; i <= 6; i++ ){
			weekly_population.setElementAt(
					new DailyPopulation( d1.addDays( i ) ), i );
			// System.out.println(
			// "WeeklyPopulation::initializeWeeklyPopulation" +
			// weekly_population.elementAt( i ).DateAndTime.getyyyy_MM_dd() );
		}
	}


	/**
	 * 時間帯ごとの聖地人リストに，名前が<code>name</code>であるキャラクタを加えます． <code>date</code>には、キルログの出た時刻を指定します。
	 * リストの日付（DateAndTime）の中に指定した日付<code>date</code>がなければ、何もしません。
	 * 当該キャラクタがリストに無かった場合のみ，リストに追加を行います．
	 * 
	 * @param name
	 *        追加するキャラクタの名前
	 * @param date
	 *        キャラクタ<code>name</code>が聖地でプレイしていることが確認された日時。
	 */
	public void add( String name, DateEx date ){
		if( date.weekStart().ymd() != weekly_population.elementAt( 0 ).pile.ymd() ){
			return;
		}else{
			int dow = date.dayOfWeek();
			int mins_of_day = date.hour() * 60 + date.minute();
			//double index_d = ((double)mins_of_day) / ((double)tantra.DAY_INTERVAL );
			int index = mins_of_day / tantra.DAY_INTERVAL;
			add( name, dow, index );
		}
	}
	private void add( String name, int dow, int index ){
		//int i, ymd, index;
		//ymd = date.ymd();
		//index = -1;
		//for( i = 0; i <= 6; i++ ){
//			if( weekly_population.elementAt( i ).pile.ymd() == ymd ){
				//index = i;
				//break;
			//}
		//}

		//int mins_of_day = date.hour() * 60 + date.minute();
		//int ind = (mins_of_day / tantra.DAY_INTERVAL);
		// System.out.println( "addPerson;ind=" + ind );

		// キャラクタが既に登録されていないかどうか検査
		int i;
		for( i = 0; i < weekly_population.elementAt( dow ).dat
				.elementAt( index ).persons.size(); i++ ){
			if( weekly_population.elementAt( dow ).dat.elementAt( index ).persons
					.elementAt( i ).equals( name ) ){
				return;
			}
		}
		weekly_population.elementAt( dow ).dat.elementAt( index ).persons
				.add( name );
	}


	/**
	 * 前回実行時のweekly_population配列の内容をファイルから読み込み，引き継ぎます
	 */
	public void read(){
		int i, j, k, length;
		String fname = tantra.settings.total_data_location + File.separator
				+ DAILY_POPULATION;
		String line;
		String token[];
		if( !tantra.exists( fname + ".txt" ) ) return;
		try{
			BufferedReader sr = new BufferedReader( new InputStreamReader(
					new FileInputStream( fname + ".txt" ), tantra.ShiftJIS ) );
			sr.readLine();		//ヘッダー "[START]"
			initialize( new DateEx() );
			for( i = 0; i <= 6; i++ ){
				int year, month, day;
				int index, dow;
				line = sr.readLine();
				// System.out.println( "i=" + i + ";line=" + line );
				token = line.split( "," );
				if( token.length - 1 < 2 ){
					sr.close();
					return;
				}
				year = Integer.valueOf( token[0] );
				month = Integer.valueOf( token[1] );
				day = Integer.valueOf( token[2] );
				weekly_population.elementAt( i ).pile = new DateEx( year, month, day, 0, 0, 0 );
				for( j = 0; j <= tantra.DAY_DEVIDE; j++ ){
					line = sr.readLine();
					// System.out.println( "j=" + j + ";line=" + line );
					token = line.split( "," );
					length = Integer.valueOf( token[0] ) - 1;
					dow = Integer.valueOf( token[1] );
					index = Integer.valueOf( token[2] );
					if( length >= 0 ){
						//DateEx dd;
						//int mins_of_day = j;
						//hour = j / 60;
						//minute = mins_of_day - hour * 60;
						for( k = 0; k <= length; k++ ){
							line = sr.readLine();
						//	dd = new DateEx( year, month, day, hour,
						//			minute + 1, 0 );
						//	add( line, dd );
							add( line, dow, index );
						}
						
					}
				}
			}
			sr.close();
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * この関数が呼ばれた時点での，weekly_population配列の内容をファイルに保存します
	 */
	public void write(){
		int i, j, k, length;
		String fname = tantra.settings.total_data_location + File.separator
				+ DAILY_POPULATION;
		if( tantra.exists( fname + ".txt" ) ){
			File fi = new File( fname + "_old.txt" );
			if( fi.exists() ){
				fi.delete();
			}
			new File( fname + ".txt" )
					.renameTo( new File( fname + "_old.txt" ) );
		}
		try{
			BufferedWriter sw = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( tantra.settings.total_data_location
							+ File.separator + DAILY_POPULATION + ".txt" ),
					tantra.ShiftJIS ) );
			sw.write( "[START]" );
			sw.newLine();
			DailyPopulation dp;
			DateEx d;
			for( i = 0; i <= 6; i++ ){
				dp = weekly_population.elementAt( i );
				d = dp.pile;
				sw.write( d.year() + "," + d.month() + "," + d.day() );
				sw.newLine();
				for( j = 0; j <= tantra.DAY_DEVIDE; j++ ){
					if( weekly_population.elementAt( i ).dat.elementAt( j ).persons
							.size() - 1 <= 0 ){
						length = -1;
					}else{
						length = weekly_population.elementAt( i ).dat
								.elementAt( j ).persons.size() - 1;
					}
					sw.write( (length + 1) + "," + i + "," + j );
					sw.newLine();
					if( length >= 0 ){
						for( k = 0; k <= length; k++ ){
							sw.write( weekly_population.elementAt( i ).dat
									.elementAt( j ).persons.elementAt( k ) );
							sw.newLine();
						}
					}
				}
			}
			sw.write( "[END]" );
			sw.newLine();
			sw.close();
		}catch( Exception e ){
			tantra.printError( e );
		}
	}


	/**
	 * DailyPopulationのデータを元に，グラフを描画したPNGファイルを作成します．
	 * @param dow
	 *        どの曜日のPNGファイルを作成するかを指定。日曜日が0、土曜日が6
	 */
	public void createPng( int dow ){
		int max_population;
		int i;
		String png_path, png_name;

		// 描こうとしているグラフの日付
		DateEx d = weekly_population.elementAt( dow ).pile;
		DateEx plot_date = new DateEx( d.year(), d.month(), d.day(), 0, 0, 0 );

		// DailyPopulation d = weekly_population.elementAt( dow );
		png_name = "picture_" + tantra.settings.target_kruma + "_"
				+ plot_date.toString( Format.yyyyMMdd ) + ".png";
		png_path = tantra.settings.html_location + File.separator + png_name;

		// 作成しようとしているファイルがfix指定か、もしくは日付が古すぎる場合。何もしない
		if( tantra.fixed.isFixed( png_name ) || plot_date.year() < 2000 ){
			return;
		}

		// 人口の最大値を検索
		max_population = -1;
		for( i = 0; i <= tantra.DAY_DEVIDE; i++ ){
			max_population = Math.max( max_population, weekly_population
					.elementAt( dow ).dat.elementAt( i ).count() );
		}

		// bmpサイズの指定とGraphicsオブジェクトの取得
		int width, height;
		width = (int) (PIXCELS_PER_X * 24 + SPACE_LEFT + SPACE_RIGHT);
		height = (int) (((int) ((max_population + 10) / 10.0) * 10.0)
				* PIXCELS_PER_Y + SPACE_UP + SPACE_DOWN);
		BufferedImage bmp = new BufferedImage( width, height,
				BufferedImage.TYPE_INT_RGB );
		Graphics2D g = bmp.createGraphics();

		// ブラシ・ペンを設定
		Color black = new Color( 0, 0, 0 );
		Color white = new Color( 255, 255, 255 );
		g.setColor( white );
		g.fillRect( 0, 0, width + 1, height + 1 );

		// 日付を記入
		Font fnt_verdana = new Font( "Verdana", Font.BOLD, 14 );
		g.setFont( fnt_verdana );
		setAntialiasing( g, true );
		g.setColor( black );
		g.drawString( plot_date.toString( Format.yyyy_MM_dd ) + "  ("
				+ DAY_OF_WEEK[plot_date.dayOfWeek()] + ")", 10, 20 );

		// グラフの背景を暗めの色で塗りつぶし
		setAntialiasing( g, false );
		g.setColor( BACKGROUND_UNKNOWN );
		g.fillRect( SPACE_LEFT, SPACE_UP, width - SPACE_LEFT - SPACE_RIGHT,
				height - SPACE_UP - SPACE_DOWN );

		// UWSCの稼動期間のみ背景を明るくする
		g.setColor( BACKGROUND_COLOR );
		for( i = 0; i < tantra.running_info.weekly_running_info.size(); i++ ){
			int start, end, from, to;
			DateEx dfrom, dto;
			dfrom = tantra.running_info.weekly_running_info.elementAt( i ).from;
			dto = tantra.running_info.weekly_running_info.elementAt( i ).to;
			if( dfrom.isNull() ){
				from = 0;
			}else{
				from = (int) (dfrom.subtract( plot_date ) / 60.0);
			}
			if( dto.isNull() ){
				to = 24 * 60;
			}else{
				to = (int) (dto.subtract( plot_date ) / 60.0);
			}

			// 描く範囲を計算
			start = xCoordinateFromMin( from );
			end = xCoordinateFromMin( to );
			// はみ出してたら調整
			if( width - SPACE_RIGHT < end ){
				end = width - SPACE_RIGHT;
			}
			if( start < SPACE_LEFT ){
				start = SPACE_LEFT;
			}
			// 塗りつぶし
			g.fillRect( start, SPACE_UP, end - start + 1, height - SPACE_UP
					- SPACE_DOWN );
		}

		// グラフの枠を描く
		g.setColor( black );
		g.drawLine( SPACE_LEFT, SPACE_UP, SPACE_LEFT, height - SPACE_DOWN );
		g.drawLine( SPACE_LEFT, height - SPACE_DOWN, width - SPACE_RIGHT,
				height - SPACE_DOWN );
		g.drawLine( width - SPACE_RIGHT, height - SPACE_DOWN, width
				- SPACE_RIGHT, SPACE_UP );
		g.drawLine( width - SPACE_RIGHT, SPACE_UP, SPACE_LEFT, SPACE_UP );

		// メモリ線とメモリ数字を書く
		Font fnt_gothic = new Font( "Verdana", Font.PLAIN, 12 );
		int x;
		g.setFont( fnt_gothic );
		setAntialiasing( g, false );
		// 横軸
		for( i = 0; i <= 24; i++ ){
			x = xCoordinateFromMin( i * 60 );
			g.drawLine( x, height - SPACE_DOWN, x, height - SPACE_DOWN
					+ LEN_MEMORI );
			g.drawString( "" + i, x - 4, height - SPACE_DOWN + 2 * LEN_MEMORI
					+ 8 );
		}
		// 縦軸
		int y;
		for( i = 0; i <= (int) ((max_population + 10) / 10.0); i++ ){
			y = height - SPACE_DOWN - (int) (i * PIXCELS_PER_Y * 10.0);
			g.drawLine( SPACE_LEFT - LEN_MEMORI, y, SPACE_LEFT, y );
			g.drawString( "" + (i * 10), SPACE_LEFT - 2 * LEN_MEMORI - 17,
					y - 5 + 10 );
		}

		// グラフを描く（全数）
		BasicStroke st20 = new BasicStroke( 2.0f );
		// Dim pen_all As New Pen(Color.Black, 2.0F)
		int last_x = -10, last_y = 0;
		int points_x[] = new int[tantra.DAY_DEVIDE + 1];
		int points_y[] = new int[tantra.DAY_DEVIDE + 1];
		setAntialiasing( g, true );
		// uwsc_date = tantra.last_entry.date();
		int timely_persons;
		DateEx tmpd = new DateEx();
		tmpd.now();
		for( i = 0; i <= tantra.DAY_DEVIDE; i++ ){
			if( tmpd.largerOrEqual( plot_date
					.addMinutes( (i * tantra.DAY_INTERVAL) ) ) ){
				// x = (int) (SPACE_LEFT + 4 + i * PIXCELS_PER_X);
				x = xCoordinateFromMin( (int)(((double)i + 0.5) * tantra.DAY_INTERVAL) );
				timely_persons = weekly_population.elementAt( dow ).dat
						.elementAt( i ).count();
				y = height - SPACE_DOWN
						- (int) (PIXCELS_PER_Y * (double) (timely_persons));
				points_x[i] = x;
				points_y[i] = y;
				last_x = x;
				last_y = y;
			}else{
				int j;
				for( j = i; j <= tantra.DAY_DEVIDE; j++ ){
					points_x[j] = last_x;
					points_y[j] = last_y;
				}
				break;
			}
		}
		g.setStroke( st20 );
		g.drawPolyline( points_x, points_y, tantra.DAY_DEVIDE + 1 );

		// グラフを描く（主神別）
		int persons[] = { 0, 0, 0 };
		int y_brahma, y_vishnu, y_shiva;
		int last_y_brahma = 0, last_y_vishnu = 0, last_y_shiva = 0;
		BasicStroke st17 = new BasicStroke( 1.7f );
		Color pen_brahma = new Color( 255, 215, 0, 200 ); // As New
		// Pen(Color.FromArgb(200,
		// Color.Gold), 1.7)
		Color pen_vishnu = new Color( 255, 0, 0, 255 ); // As New
		// Pen(Color.FromArgb(200,
		// Color.Red), 2.0)
		Color pen_shiva = new Color( 7, 62, 248, 200 ); // As New
		// Pen(Color.FromArgb(200,
		// 7, 62, 248), 2.0)
		BasicStroke st25 = new BasicStroke( 2.5f ); // Dim pen_back As New
		// Pen(Color.Black, 2.5)
		int points_brahma_x[] = new int[tantra.DAY_DEVIDE + 1];
		int points_brahma_y[] = new int[tantra.DAY_DEVIDE + 1];
		int points_vishnu_x[] = new int[tantra.DAY_DEVIDE + 1];
		int points_vishnu_y[] = new int[tantra.DAY_DEVIDE + 1];
		int points_shiva_x[] = new int[tantra.DAY_DEVIDE + 1];
		int points_shiva_y[] = new int[tantra.DAY_DEVIDE + 1];
		tmpd.now();
		last_x = -10;
		for( i = 0; i <= tantra.DAY_DEVIDE; i++ ){
			if( tmpd.largerOrEqual( plot_date.addMinutes( i
					* tantra.DAY_INTERVAL ) ) ){
				createPngCount( weekly_population.elementAt( dow ).dat
						.elementAt( i ), persons, tantra.kruma );
				x = xCoordinateFromMin( (int)(((double)i + 0.5) * tantra.DAY_INTERVAL) );
				y_brahma = (int) (height - SPACE_DOWN - PIXCELS_PER_Y
						* persons[0]);
				y_vishnu = (int) (height - SPACE_DOWN - PIXCELS_PER_Y
						* persons[1]);
				y_shiva = (int) (height - SPACE_DOWN - PIXCELS_PER_Y
						* persons[2]);
				points_brahma_x[i] = x;
				points_brahma_y[i] = y_brahma;
				points_vishnu_x[i] = x;
				points_vishnu_y[i] = y_vishnu;
				points_shiva_x[i] = x;
				points_shiva_y[i] = y_shiva;
				last_x = x;
				last_y_brahma = y_brahma;
				last_y_vishnu = y_vishnu;
				last_y_shiva = y_shiva;
			}else{
				int j;
				for( j = i; j <= tantra.DAY_DEVIDE; j++ ){
					points_brahma_x[j] = last_x;
					points_brahma_y[j] = last_y_brahma;
					points_vishnu_x[j] = last_x;
					points_vishnu_y[j] = last_y_vishnu;
					points_shiva_x[j] = last_x;
					points_shiva_y[j] = last_y_shiva;
				}
				break;
			}
		}
		setAntialiasing( g, true );
		g.setColor( black );
		g.setStroke( st25 );
		g
				.drawPolyline( points_brahma_x, points_brahma_y,
						tantra.DAY_DEVIDE + 1 );
		g.setColor( white );
		g.setStroke( st17 );
		g
				.drawPolyline( points_brahma_x, points_brahma_y,
						tantra.DAY_DEVIDE + 1 );
		g.setColor( pen_brahma );
		g.setStroke( st17 );
		g
				.drawPolyline( points_brahma_x, points_brahma_y,
						tantra.DAY_DEVIDE + 1 );
		g.setColor( pen_vishnu );
		g.setStroke( st20 );
		g
				.drawPolyline( points_vishnu_x, points_vishnu_y,
						tantra.DAY_DEVIDE + 1 );
		g.setColor( pen_shiva );
		g.drawPolyline( points_shiva_x, points_shiva_y, tantra.DAY_DEVIDE + 1 );

		// ファイルに保存
		try{
			BufferedOutputStream sw = new BufferedOutputStream(
					new FileOutputStream( png_path ) );
			ImageIO.write( bmp, "png", sw );
			sw.close();
			tantra.uploader_main.registerUpload( png_path );
		}catch( Exception e ){
			tantra.printError( e );
		}

		// オブジェクトの破棄
		g.dispose();
	}


	/**
	 * 午前0時からの分数から、グラフに描くべきx座標を計算します
	 * @param minute_of_day
	 * 		午前0時からの分数
	 * @return
	 * 		グラフに描くべきx座標
	 */
	private static int xCoordinateFromMin( int minute_of_day ){
		return (int) (SPACE_LEFT + minute_of_day / 60.0 * PIXCELS_PER_X);
	}


	/**
	 * 描画モードを切り替えます
	 * @param g
	 * 		切り替え操作の対象となる<code>Graphics2D</code>オブジェクト
	 * @param mode
	 * 		アンチエリアシングをONにする場合はtrueを、OFFにするばあいはfalseを指定します
	 */
	private static void setAntialiasing( Graphics2D g, boolean mode ){
		if( mode ){
			g.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON );
		}else{
			g.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_OFF );
		}
	}


	/**
	 * 主神ごとの人口を調べます。
	 * 
	 * @param d
	 *        調査対象のDailyPopulationPart
	 * @param result
	 *        主神ごとの人口を格納する配列。
	 * @param kruma
	 *        主神判定に使用するPersonalDataの配列。普通はtantra.krumaを使う。
	 */
	public static void createPngCount( DailyPopulationPart d, int result[],
			Vector<PersonalData> kruma ){
		result[0] = 0;
		result[1] = 0;
		result[2] = 0;
		if( d.persons.size() <= 0 ){
			return;
		}
		int i, index, length;
		length = d.persons.size() - 1;
		for( i = 0; i <= length; i++ ){
			index = PersonalData.indexOf( kruma, d.persons.elementAt( i ),
					false );
			if( index >= 0 ){
				switch( kruma.elementAt( index ).detectLord() ){
					case Brahma:
						result[0] += 1;
						break;
					case Vishnu:
						result[1] += 1;
						break;
					case Shiva:
						result[2] += 1;
				}
			}
		}
	}


	/**
	 * 現在データ保持している週の第dow日目の日付を返します。
	 * 
	 * @param dow
	 * 		取り出したい曜日。日曜日=>0、土曜日=>6
	 * @return
	 * 		現在データを保持している週の第dow日目の日付
	 */
	public DateEx currentDate( int dow ){
		return weekly_population.elementAt( dow ).pile;
	}


	/**
	 * 聖地人口を掲載したHTMLを作成する
	 */
	public void createHtml(){
		DateEx week_start = weekly_population.elementAt( 0 ).pile.weekStart();
		String fpath = tantra.settings.html_location + File.separator
				+ "population_" + tantra.settings.target_kruma + "_"
				+ week_start.toString( Format.yyyyMMdd ) + ".html";
		if( !tantra.fixed.isFixed( fpath ) ){
			try{
				BufferedWriter unit = new BufferedWriter(
						new OutputStreamWriter( new FileOutputStream( fpath ),
								tantra.ShiftJIS ) );
				Html.writeHeader( unit, "聖地人口 "
						+ week_start.toString( Format.yyyy_MM_dd ) );
				unit.write( "<body>" );
				unit.newLine();
				String fname;
				String navigation;
				navigation = "<p>"
						+ Html.LINK_TOP
						+ Html.GT
						+ Html.tagA( "index_" + tantra.settings.target_kruma
								+ ".html", tantra.settings.target_kruma )
						+ Html.GT + week_start.toString( Format.yyyy_MM_dd )
						+ Html.GT + "population" + "</p>";
				unit.write( navigation );
				unit.newLine();
				int i;
				for( i = 0; i <= 6; i++ ){
					fname = "picture_"
							+ tantra.settings.target_kruma
							+ "_"
							+ week_start.addDays( i )
									.toString( Format.yyyyMMdd ) + ".png";
					if( new File( tantra.settings.html_location
							+ File.separator + fname ).exists() ){
						unit.write( "<img src=\""
								+ fname
								+ "\" alt=\""
								+ week_start.addDays( i ).toString(
										Format.yyyy_MM_dd )
								+ "の聖地人口時間推移のグラフ\"><br>" );
						unit.newLine();
					}else{
						break;
					}
				}
				unit.write( "<hr>" );
				unit.newLine();
				unit.write( navigation );
				unit.newLine();
				Html.addAffiliate( unit );
				unit.write( "</body>" );
				unit.newLine();
				unit.write( "</html>" );
				unit.newLine();
				unit.close();
			}catch( Exception e ){
				tantra.printError( e );
			}
			tantra.uploader_main.registerUpload( tantra.settings.html_location
					+ File.separator + "population_"
					+ tantra.settings.target_kruma + "_"
					+ week_start.toString( Format.yyyyMMdd ) + ".html" );
		}
	}
}


/**
 * 一日分のDailyPopulationPartを保持する
 * 
 * @author Luft
 */
class DailyPopulation {

	/** 人口データの保持を担当する日付 */
	public DateEx pile;
	/** 一日分の人口データを保持するための<code>DailyPopulationPart</code>の<code>Vector</code> */
	public Vector<DailyPopulationPart> dat = new Vector<DailyPopulationPart>();


	/**
	 * 指定された日付を用いて、新しい<code>DailyPopulation</code>のインスタンスを生成します。
	 * @param d
	 * 		生成されるインスタンスが担当する日付。
	 */
	public DailyPopulation( DateEx d ) {
		pile = d;
		dat.setSize( tantra.DAY_DEVIDE + 1 );
		int i;
		for( i = 0; i <= tantra.DAY_DEVIDE; i++ ){
			dat.setElementAt( new DailyPopulationPart(), i );
		}
	}
}


/**
 * 一日をtantra.DAY_DEVIDE + 1分割した時間帯において活動していたキャラクタ名のリストを保持
 * 
 * @author Luft
 */
class DailyPopulationPart {

	/** このオブジェクトが担当する時間帯において、聖地で活動が確認されたキャラクタのリスト */
	public Vector<String> persons = new Vector<String>();


	/**
	 * <code>persons</code>を初期化し、新しい<code>DailyPopulationPart</code>のインスタンスを生成します。
	 */
	public DailyPopulationPart() {
		persons.removeAllElements();
	}


	/**
	 * 登録されたキャラクタの数を返します。
	 * @return
	 * 		<code>persons</code>の要素数
	 */
	public int count(){
		return persons.size();
	}

}
