package tantra;

import java.io.*;
import java.util.*;
import tantra.DateEx.*;

/**
 * データ解析をする上で基本となる定数などを格納するほか、クラス間で共有するデータの保持を行う。
 * 
 * @author Luft
 * 
 */
public class tantra {

	/** debugモードかどうかを表すフラグ */
	public static boolean debug = false;

	/** 取り扱うテキストファイルのデフォルトのエンコーディング */
	public final static String ShiftJIS = "MS932";
	/** 解析の時間分解能を決める定数 */
	public final static int DAY_DEVIDE = 71;
	/** 人口解析の時間分解能 */
	public final static int DAY_INTERVAL = 24 * 60 / (DAY_DEVIDE + 1);

	public final static String WEEKLYRUNNINGINFO = "weekly_running_info";
	public final static String LOCKFILE_EXE = "lockfile_exe.bin";
	public final static String LOCKFILE_UWSC = "lockfile_uwsc.bin";
	public final static String UWSC_LOG = "uwsc_log.txt";
	public final static String AFFILIATE_FILE_NAME = "affiliate.txt";
	public final static String TOTAL_DATA_FILE_NAME = "tantra_data.txt";
	public final static String SETTING_FILE_NAME = "tantra_data.ini";
	public final static String JOB_LIST = "job_list.txt";
	public final static String SUSPECT_FILE_NAME = "suspect.txt";
	public final static String PRE_ANALYZED_DATA = "pre_analyzed_data.txt";
	public final static String PRE_ANALYZED_DATA_WEEKLY = "pre_analyzed_data_weekly.txt";
	public final static String PRE_ANALYZED_DATA_OLD = "pre_analyzed_data_old.txt";
	public final static String PRE_ANALYZED_DATA_OLD_WEEKLY = "pre_analyzed_data_old_weekly.txt";
	public final static String ERROR_LOG = "error_log.txt";
	/* FTP設定が保存されたファイル */
	//public final static String PASS_TXT = "pass.txt";

	/** このアプリのバージョン */
	public final static String ver = "2.0";

	public static Vector<PersonalData> kruma = new Vector<PersonalData>();
	public static Vector<PersonalData> kruma_weekly = new Vector<PersonalData>();
	public static Settings settings;
	public static UwscLogEntry last_entry = new UwscLogEntry();
	public static FixedFileList fixed;
	public static SuspectList suspects;
	public static WeeklyPopulation population;
	public static WeeklyRunningInfo running_info;
	public static Uploader uploader_main;
	public static Uploader uploader_backup;
	public static JobList joblist;

	/** total data形式のデータを解析するときのスイッチ */
	public enum AnalyzeMode {
		/** 起動時の解析時 */
		PRE_ANALYZATION,
		/** 普通にchat logフォルダを監視しているとき */
		ANOTHER
	}

	/** ウィークリーランキングを出力するときのモード */
	public enum OutputMode {
		/** DMG値と薬値をもとにランキングを出します */
		TOTAL,
		/** 薬値をもとにランキングを出力します */
		DOCTOR,
		/** 種族別のランキングを出力します。DMG値と薬値を基にします */
		JOB
	}


	/**
	 * booleanをintに変換します
	 * @param b
	 * 		boolean値
	 * @return
	 * 		b == trueであれば1を、そうでなければ1を返します
	 */
	public static int intValue( boolean b ){
		if( b ){
			return 1;
		}else{
			return 0;
		}
	}

	
	/**
	 * テスト用のデータを作成します。
	 */
	public static void createTestData(){
		String CURRENT_DIR = new File( "." ).getAbsoluteFile().getParent();
		if( CURRENT_DIR.endsWith( File.separator ) ){
			CURRENT_DIR = CURRENT_DIR.substring( 0, CURRENT_DIR.length() - 1 );
		}

		try{
			// 設定ファイルの作成
			String fpath = CURRENT_DIR + File.separator + "tantra_data.ini";
			IniFile.WritePrivateProfileString( "", "last_read_line", "", fpath );
			IniFile.WritePrivateProfileString( "", "chat_log_location", CURRENT_DIR + File.separator + "ScreenShot", fpath );
			IniFile.WritePrivateProfileString( "", "backup_location", CURRENT_DIR + File.separator + "ScreenShot"
					+ File.separator + "backup", fpath );
			IniFile.WritePrivateProfileString( "", "total_data_location", CURRENT_DIR + File.separator + "Tantra_hack", fpath );
			IniFile.WritePrivateProfileString( "", "html_location", CURRENT_DIR + File.separator + "Tantra_hack"
					+ File.separator + "html", fpath );
			IniFile.WritePrivateProfileString( "", "last_read_log", "", fpath );
			IniFile.WritePrivateProfileString( "", "target_kruma", "debug", fpath );
			IniFile.WritePrivateProfileString( "", "uwsc_location", CURRENT_DIR + File.separator + "Tantra_hack"
					+ File.separator + "uwsc", fpath );
			IniFile.WritePrivateProfileString( "", "last_zipped_date", new DateEx().toString( Format.yyyyMMdd ), fpath );
			IniFile.WritePrivateProfileString( "", "listening_interval", "" + 10000, fpath );
			IniFile.WritePrivateProfileString( "", "last_uwsc_log_entry", "" + 0L, fpath );

			IniFile.WritePrivateProfileString( "ftp_main", "host", "dumy", fpath );
			IniFile.WritePrivateProfileString( "ftp_main", "path_on_server", "dumy", fpath );
			IniFile.WritePrivateProfileString( "ftp_main", "user_id", "dumy", fpath );
			IniFile.WritePrivateProfileString( "ftp_main", "user_pw", "dumy", fpath );

			IniFile.WritePrivateProfileString( "ftp_backup", "host", "dumy", fpath );
			IniFile.WritePrivateProfileString( "ftp_backup", "path_on_server", "dumy", fpath );
			IniFile.WritePrivateProfileString( "ftp_backup", "user_id", "dumy", fpath );
			IniFile.WritePrivateProfileString( "ftp_backup", "user_pw", "dumy", fpath );

			Settings tmp = new Settings();
			tmp.read( CURRENT_DIR + File.separator + "tantra_data.ini" );

			// 上で作成した設定ファイルに即したフォルダの配置（必要があれば）
			File dir = new File( tmp.chat_log_location );
			if( !dir.exists() ) dir.mkdir();
			dir = new File( tmp.backup_location );
			if( !dir.exists() ) dir.mkdir();
			//dir = new File( tmp.edit1_location );
			//if( !dir.exists() ) dir.mkdir();
			dir = new File( tmp.total_data_location );
			if( !dir.exists() ) dir.mkdir();
			dir = new File( tmp.html_location );
			if( !dir.exists() ) dir.mkdir();
			dir = new File( tmp.uwsc_location );
			if( !dir.exists() ) dir.mkdir();

			// ダミーのFTP設定ファイルを作成
			/*BufferedWriter sw = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( CURRENT_DIR + File.separator
							+ "pass.txt" ), ShiftJIS ) );
			sw.write( "dumy" );
			sw.newLine();
			sw.write( "dumy" );
			sw.newLine();
			sw.write( "dumy" );
			sw.newLine();
			sw.write( "dumy" );
			sw.newLine();
			sw.write( "dumy" );
			sw.newLine();
			sw.write( "dumy" );
			sw.newLine();
			sw.write( "dumy" );
			sw.newLine();
			sw.write( "dumy" );
			sw.newLine();
			sw.close();*/

			// 既存の解析データを消去
			if( exists( tmp.total_data_location + File.separator
					+ WEEKLYRUNNINGINFO + ".txt" ) ){
				kill( tmp.total_data_location + File.separator
						+ WEEKLYRUNNINGINFO + ".txt" );
			}
			if( exists( tmp.total_data_location + File.separator
					+ AFFILIATE_FILE_NAME ) ){
				kill( tmp.total_data_location + File.separator
						+ AFFILIATE_FILE_NAME );
			}
			if( exists( tmp.total_data_location + File.separator
					+ TOTAL_DATA_FILE_NAME ) ){
				kill( tmp.total_data_location + File.separator
						+ TOTAL_DATA_FILE_NAME );
			}
			if( exists( tmp.total_data_location + File.separator + JOB_LIST ) ){
				kill( tmp.total_data_location + File.separator + JOB_LIST );
			}
			if( exists( tmp.total_data_location + File.separator
					+ SUSPECT_FILE_NAME ) ){
				kill( tmp.total_data_location + File.separator
						+ SUSPECT_FILE_NAME );
			}
			if( exists( tmp.total_data_location + File.separator
					+ PRE_ANALYZED_DATA_OLD ) ){
				kill( tmp.total_data_location + File.separator
						+ PRE_ANALYZED_DATA_OLD );
			}
			if( exists( tmp.total_data_location + File.separator
					+ PRE_ANALYZED_DATA_WEEKLY ) ){
				kill( tmp.total_data_location + File.separator
						+ PRE_ANALYZED_DATA_WEEKLY );
			}
			if( exists( tmp.total_data_location + File.separator
					+ PRE_ANALYZED_DATA_OLD_WEEKLY ) ){
				kill( tmp.total_data_location + File.separator
						+ PRE_ANALYZED_DATA_OLD_WEEKLY );
			}
			if( exists( tmp.total_data_location + File.separator
					+ PRE_ANALYZED_DATA ) ){
				kill( tmp.total_data_location + File.separator
						+ PRE_ANALYZED_DATA );
			}

			// ダミーのchat logを作成。
			BufferedWriter sw = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( tmp.chat_log_location
							+ File.separator + "ChatLog_2007_11_7_0.txt" ),
					ShiftJIS ) );
			sw.write( "[ChatLog]\n" );
			sw.write( "[告知]B が C に勝利しました。 (20071107,00:00:20)\n"
					+ "[告知]B が A に勝利しました。 (20071107,00:00:20)\n"
					+ "[告知]B が A に勝利しました。 (20071107,00:00:10)\n"
					+ "[告知]A が B に勝利しました。 (20071107,00:00:00)\n"
					+ "[OpenMsg]\n" );
			sw.close();

			sw = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( tmp.chat_log_location
							+ File.separator + "ChatLog_2007_11_7_1.txt" ),
					ShiftJIS ) );
			sw.write( "[ChatLog] this is dumy chat log file\n"
					+ "[告知]B が A に勝利しました。 (20071107,00:00:50)\n"
					+ "[告知]B が A に勝利しました。 (20071107,00:00:40)\n"
					+ "[告知]B が D に勝利しました。 (20071107,00:00:20)\n"
					+ "[告知]B が C に勝利しました。 (20071107,00:00:20)\n"
					+ "[告知]B が A に勝利しました。 (20071107,00:00:20)\n"
					+ "[告知]B が A に勝利しました。 (20071107,00:00:10)\n"
					+ "[告知]A が B に勝利しました。 (20071107,00:00:00)\n"
					+ "[OpenMsg]\n" + "Null\n" );
			sw.close();

			sw = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( tmp.chat_log_location
							+ File.separator + "ChatLog_2007_11_8_0.txt" ),
					ShiftJIS ) );
			sw.write( "[ChatLog] this is dumy chat log file\n"
					+ "[告知]B が A に勝利しました。 (20071108,00:00:50)\n"
					+ "[告知]B が A に勝利しました。 (20071108,00:00:40)\n"
					+ "[告知]B が D に勝利しました。 (20071108,00:00:20)\n"
					+ "[告知]B が C に勝利しました。 (20071108,00:00:20)\n"
					+ "[告知]B が A に勝利しました。 (20071108,00:00:20)\n"
					+ "[告知]B が A に勝利しました。 (20071108,00:00:10)\n"
					+ "[告知]A が B に勝利しました。 (20071108,00:00:00)\n"
					+ "[OpenMsg]\n" + "Null\n" );
			sw.close();

			// ダミーのuwsc logデータを作成
			sw = new BufferedWriter( new OutputStreamWriter(
					new FileOutputStream( tmp.uwsc_location + File.separator
							+ UWSC_LOG ), ShiftJIS ) );
			sw.write( "1,20071108,22:31:20" );
			sw.newLine();
			sw.write( "1,20071108,22:31:21" );
			sw.newLine();
			sw.write( "1,20071108,22:33:42" );
			sw.newLine();
			sw.write( "2,20071108,22:35:47" );
			sw.newLine();
			sw.write( "0,20071108,22:37:17" );
			sw.newLine();
			sw.write( "1,20071108,22:39:21" );
			sw.newLine();
			sw.write( "1,20071108,22:41:27" );
			sw.newLine();
			sw.write( "2,20071108,22:43:31" );
			sw.newLine();
			sw.write( "0,20071108,22:45:36" );
			sw.newLine();
			sw.write( "1,20071108,22:47:41" );
			sw.newLine();
			sw.write( "1,20071008,22:49:46" );
			sw.newLine();
			sw.write( "1,20071108,22:51:51" );
			sw.newLine();
			sw.write( "0,20071108,22:53:56" );
			sw.newLine();
			sw.write( "1,20071108,22:56:01" );
			sw.newLine();
			sw.write( "1,20071108,22:58:06" );
			sw.newLine();
			sw.close();

		}catch( Exception e ){
			printError( e );
		}

	}


	/**
	 * コンソールに日付つきのメッセージを表示します
	 * 
	 * @param line
	 *        表示する行
	 */
	public static void pushConsoleLine( String line ){
		DateEx d = new DateEx();
		d.now();
		System.out.println( d.toString( Format.yyyy_MM_dd ) + " "
				+ d.toString( Format.hh_mm_ss ) + ";" + line );
	}


	/**
	 * ファイルが存在するか否かを確認します
	 * 
	 * @param fpath
	 *        確認するファイルの名前
	 * @return 存在すればtrueを、そうでなければfalseを返します。
	 */
	public static boolean exists( String fpath ){
		File fp = new File( fpath );
		return fp.exists();
	}


	/**
	 * ファイルを削除します
	 * 
	 * @param fpath
	 *        削除するファイルの名前
	 * @return 削除に成功すればtrueを、そうでなければfalseを返します。
	 */
	public static boolean kill( String fpath ){
		File fp = new File( fpath );
		return fp.delete();
	}


	/**
	 * ファイルの名前を変更します
	 * 
	 * @param source
	 *        変更前のファイルの名前
	 * @param dest
	 *        変更後のファイルの名前
	 * @return 変更に成功すればtrueを、そうでなければfalseを返します。
	 */
	public static boolean rename( String source, String dest ){
		File fp = new File( source );
		File fpdest = new File( dest );
		return fp.renameTo( fpdest );
	}


	/**
	 * 指定した日付に開発者が残したメッセージを取り出します。
	 * 
	 * @param y
	 *		年
	 * @param m
	 *		月
	 * @param d
	 *      日
	 * @return
	 * 		指定された日付に記録されたメッセージがあればそれを、メッセージがなければ""を返します。
	 */
	public static String getDeveloppersComments( int y, int m, int d ){
		// [${author}]\n
		// ${comment_1}\n
		// ...
		// ${comment_n}\n
		// のフォーマットで追加すること。
		int ymd = y * 10000 + m * 100 + d;
		switch( ymd ){
			case 20071115:
				return "[Luft]\n人がいない場合でも人口のグラフを作成するように改善。\n" +
					"Exception catch時に内容をファイルを保存する機能を追加。\n" +
					"聖地1で稼動開始\n" +
					"todo list\n" +
					"種族データを保持するTribeServerを作る。vishnu_lowとvishnu_highで種族データを共有。\n" +
					"classs tantra_dataのwaitEditingに、応答メッセージのような一時ファイル（たとえば\"aye ma'am\"とかｗ）を作成する機能を追加。\n";
			case 20071112:
				return "[Luft]\nようやくftpが稼動するようになった。\n聖地2を人柱に稼動テスト。\n";
			case 20071107:
				return "[Luft]\nString.substringの使い方完全に間違ってた・・・。\n\\nをCR+LFだと思ってた・・・。\n";
			case 20071106:
				return "[Luft]\n一応動くバージョンが完成。\n";
			case 20071030:
				return "[Luft]\nJAVA化開始。\n";
			case 20071101:
				return "[Luft]\n本日までに書いた関数など\n"
						+ "tantra_data.Init()\n"
						+ "tantra_data.LoadDump()\n"
						+ "tantra_data.LoadJobList(String)\n"
						+ "tantra_data.LoadPreAnalyzedData()\n"
						+ "tantra_data.LoadPreAnalyzedDataCheckCorrect(String, String)\n"
						+ "tantra_data.LoadPreAnalyzedDataLoad(Vector<PersonalData>, String)\n"
						+ "tantra_data.main(String[])\n"
						+ "tantra_data.OnLoad()\n"
						+ "tantra_data.PushConsoleLine(String)\n"
						+ "tantra_data.ReadFTPSettings()\n"
						+ "tantra_data.TreatLine(String)\n"
						+ "tantra.ChatLogServer.ChatLogServer(String, String, String, String)\n"
						+ "tantra.ChatLogServer.GetChatLogLine(String, Vector<String>)\n"
						+ "tantra.ChatLogServer.GetDateFromLine(String)\n"
						+ "tantra.ChatLogServer.GetNextLog()\n"
						+ "tantra.ChatLogServer.isInitialized()\n"
						+ "tantra.ChatLogServer.OldestFileName(String)\n"
						+ "tantra.Filter.accept(File, String)\n"
						+ "tantra.DateEx.DATE_NULL\n"
						+ "tantra.DateEx.FromyyyyMMdd(String)\n"
						+ "tantra.DateEx.GetDateFromLine(String)\n"
						+ "tantra.DateEx.LineDate(String)\n"
						+ "tantra.DateEx.DateEx()\n"
						+ "tantra.DateEx.DateEx(int, int, int, int, int, int)\n"
						+ "tantra.DateEx.AddDays(int)\n"
						+ "tantra.DateEx.AddMinutes(int)\n"
						+ "tantra.DateEx.day()\n"
						+ "tantra.DateEx.DayOfWeek()\n"
						+ "tantra.DateEx.equals(DateEx)\n"
						+ "tantra.DateEx.Gethh_mm_ss()\n"
						+ "tantra.DateEx.GetSumYMD()\n"
						+ "tantra.DateEx.GetWeekStart()\n"
						+ "tantra.DateEx.Getyyyy_MM_dd()\n"
						+ "tantra.DateEx.GetyyyyMMdd()\n"
						+ "tantra.DateEx.hour()\n"
						+ "tantra.DateEx.IsNull()\n"
						+ "tantra.DateEx.LargerOrEqual(DateEx)\n"
						+ "tantra.DateEx.LargerThan(DateEx)\n"
						+ "tantra.DateEx.minute()\n"
						+ "tantra.DateEx.month()\n"
						+ "tantra.DateEx.Now()\n"
						+ "tantra.DateEx.second()\n"
						+ "tantra.DateEx.Subtract(DateEx)\n"
						+ "tantra.DateEx.year()\n"
						+ "tantra.FileSpecifier.CompareFileSpecifier(FileSpecifier, FileSpecifier)\n"
						+ "tantra.FileSpecifier.GetFileSpecifier(String)\n"
						+ "tantra.FileSpecifier.IsEqual(FileSpecifier, FileSpecifier)\n"
						+ "tantra.FileSpecifier.FileSpecifier()\n"
						+ "tantra.FileSpecifier.FileSpecifier(int, int, int, int)\n"
						+ "tantra.FixedFileList.FixedFileList(Settings)\n"
						+ "tantra.FixedFileList.Fix(String)\n"
						+ "tantra.FixedFileList.IsFixed(String)\n"
						+ "tantra.FixedFileList.LoadFixedFileList()\n"
						+ "tantra.FixedFileList.SetFilesAsFixed(DateEx)\n"
						+ "tantra.FTPSetting.FTPSetting()\n"
						+ "tantra.FTPSetting.FTPSetting(String, String, String, String)\n"
						+ "tantra.Job.JobFromString(String)\n"
						+ "tantra.Job.JobToString(int)\n"
						+ "tantra.Lord.LordToString(int)\n"
						+ "tantra.Lord.LordToStringEn(int)\n"
						+ "tantra.Path.GetDirectoryName(String)\n"
						+ "tantra.Path.GetExtension(String)\n"
						+ "tantra.Path.GetFileName(String)\n"
						+ "tantra.Path.GetFileNameWithoutExtension(String)\n"
						+ "tantra.Path.GetPathRoot(String)\n"
						+ "tantra.Path.test()\n"
						+ "tantra.PersonalData.IndexOfKruma(Vector<PersonalData>, String, boolean)\n"
						+ "tantra.PersonalData.ManageLordValue(PersonalData, PersonalData)\n"
						+ "tantra.PersonalData.PrintPersonalData(Vector<PersonalData>, String, String)\n"
						+ "tantra.PersonalData.PersonalData(String)\n"
						+ "tantra.PersonalData.PersonalData(String, boolean)\n"
						+ "tantra.PersonalData.CalculateActivityTime(DateEx)\n"
						+ "tantra.PersonalData.DetectLord()\n"
						+ "tantra.PersonalData.DMG()\n"
						+ "tantra.PersonalData.Doctor()\n"
						+ "tantra.PersonalData.IsEqual(PersonalData)\n"
						+ "tantra.PersonalData.name()\n"
						+ "tantra.PersonalData.RatingValue(Mode)\n"
						+ "tantra.PersonalData.SumsLordValue()\n"
						+ "tantra.PersonalData.WinningPercentage()\n"
						+ "tantra.Settings.Settings()\n"
						+ "tantra.Settings.ReadSettingFile(String)\n"
						+ "tantra.Settings.WriteSettingFile(String)\n"
						+ "tantra.SuspectList.SuspectList(String)\n"
						+ "tantra.SuspectList.IsSuspect(String)\n"
						+ "tantra.SuspectList.LoadSuspectList()\n"
						+ "tantra.tantra.GetDeveloppersComments(int, int, int)\n"
						+ "tantra.tantra.msplit(String, String)\n"
						+ "tantra.Uploader.Uploader(FTPSetting)\n"
						+ "tantra.Uploader.Uploader(String, String, String, String)\n"
						+ "tantra.Uploader.RegisterUpload(String)\n"
						+ "tantra.Uploader.TryFileUpload(Vector<String>)\n"
						+ "tantra.Uploader.TryUpload()\n"
						+ "tantra.UwscLogEntry.UwscLogEntry()\n"
						+ "tantra.UwscLogEntry.date()\n"
						+ "tantra.UwscLogEntry.UwscIsEqual(UwscLogEntry)\n"
						+ "tantra.DailyPopulation.DailyPopulation(DateEx)\n"
						+ "tantra.DailyPopulationPart.DailyPopulationPart()\n"
						+ "tantra.DailyPopulationPart.countPersons()\n"
						+ "tantra.WeeklyPopulation.MakePng(DailyPopulation, Vector<PersonalData>)\n"
						+ "tantra.WeeklyPopulation.MakePngCountPersonsLord(DailyPopulationPart, int[], Vector<PersonalData>)\n"
						+ "tantra.WeeklyPopulation.WeeklyPopulation(Settings)\n"
						+ "tantra.WeeklyPopulation.AddPerson(String, DateEx)\n"
						+ "tantra.WeeklyPopulation.InitializeWeeklyPopulation(DateEx)\n"
						+ "tantra.WeeklyPopulation.LoadDailyPopulation()\n"
						+ "tantra.WeeklyPopulation.WriteDailyPopulation()\n"
						+ "tantra.WeeklyRunningInfo.DailyRunningInfo.DailyRunningInfo(DateEx)\n"
						+ "tantra.WeeklyRunningInfo.RunningInfoEntry.RunningInfoEntry(DateEx, DateEx)\n"
						+ "tantra.WeeklyRunningInfo.WeeklyRunningInfo(Settings)\n"
						+ "tantra.WeeklyRunningInfo.AddRunningInfoEntryEnds(DateEx)\n"
						+ "tantra.WeeklyRunningInfo.AddRunningInfoEntryStarts(DateEx)\n"
						+ "tantra.WeeklyRunningInfo.InitializeWeeklyRunningInfo(DateEx)\n"
						+ "tantra.WeeklyRunningInfo.LoadDailyRunningInfo()\n"
						+ "tantra.WeeklyRunningInfo.WriteDailyRunningInfo()\n";
			case 20071105:
				return "[Luft]\nftpでアップロードできないよ。\n" + "cvsサーバーを設置。\n"
						+ "docをネットに上げた。";
			default:
				return "";
		}
		// return "";
	}


	public static void fileCopy( String src, String dest ) throws IOException{
		int buffer_size = 2048;
		byte[] buffer = new byte[buffer_size];

		InputStream in = null;
		OutputStream out = null;
		try{
			// ファイルのオープン & ストリームのバッファサイズ指定
			in = new BufferedInputStream( new FileInputStream( src ),
					buffer_size );
			out = new BufferedOutputStream( new FileOutputStream( dest ),
					buffer_size );

			// コピー実行
			int readsize;
			while( (readsize = in.read( buffer )) != -1 ){
				out.write( buffer, 0, readsize );
			}
		}finally{
			// ファイルクローズ
			try{
				if( in != null ){
					in.close();
				}
			}catch( IOException e ){
				printError( e );
			}
			try{
				if( out != null ){
					out.close();
				}
			}catch( IOException e ){
				printError( e );
			}
		}
	}


	/**
	 * エラーをログファイルに書き込みます
	 * @param ex
	 * 		エラー情報
	 */
	public static void printError( Exception ex ){
		printError( ex, "" );
	}

	public static void printError( Exception ex, String message ){
		ex.printStackTrace();
		StringWriter stringWriter = new StringWriter();
        PrintWriter printWriter = new PrintWriter( stringWriter );
        ex.printStackTrace( printWriter );
        printError( stringWriter.toString() + message );
	}

	public static void printError( String message ){
		try{
	        DateEx n = new DateEx();
			n.now();
			BufferedWriter sw = 
				new BufferedWriter( 
						new OutputStreamWriter( 
								new FileOutputStream( ERROR_LOG, true ), ShiftJIS ) );
			sw.write( n.toString() );
			sw.newLine();
			sw.write( message );
			sw.newLine();
			sw.close();
		}catch( Exception e ){
			e.printStackTrace();
		}
	}
}
